@@ -499,9 +499,7 @@ impl PieceTrait for Clock {
       let pointer = Html::lit(r##"
   pointer-events="none"
              "##);
-      let font = Html::lit(r##"
-  font-family="Latin Modern Mono, monospace" font-size="6" font-weight="700"
-             "##);
+      let font = monospace_font(6);
       hwrite!(f, r##"
   <text x="1" y="{}" {} {} fill="{}" >{}{}{}</text>"##,
              y, font, pointer, Html::lit(show.text),
@@ -640,7 +638,7 @@ impl PieceTrait for Clock {
       },
       "reset" => {
         if state.current.is_some() {
-          throw!(OE::BadPieceStateForOperation);
+          throw!(Ia::BadPieceStateForOperation);
         }
         state.reset(&self.spec);
         (Unpredictable, format!("reset"))
@@ -648,7 +646,7 @@ impl PieceTrait for Clock {
       "claim-x" | "claim-y" => {
         let user = get_user();
         if let Some(_gpl) = gs.players.get(state.users[user].player) {
-          throw!(OE::BadPieceStateForOperation);
+          throw!(Ia::BadPieceStateForOperation);
         }
         state.users[user].player = player;
         if state.users[! user].player == player {
@@ -663,7 +661,7 @@ impl PieceTrait for Clock {
         (Unpredictable, format!("cleared player {} at the", user))
       },
       _ => {
-        throw!(OE::BadPieceStateForOperation);
+        throw!(Ia::BadPieceStateForOperation);
       }
     };
 
