@@ -5,31 +5,19 @@
 use crate::prelude::*;
 
 #[derive(Error,Debug)]
-pub enum OnlineError {
+pub enum Fatal { // Includes _bogus_ client updates, see PROTOCOL.md
   #[error("Game in process of being destroyed")]
   GameBeingDestroyed(#[from] GameBeingDestroyed),
   #[error("client session not recognised (terminated by server?)")]
   NoClient,
   #[error("player not part of game (removed?)")]
   NoPlayer(#[from] PlayerNotFound),
-  #[error("invalid Z coordinate")]
-  InvalidZCoord,
   #[error("Server operational problems - consult administrator: {0}")]
   ServerFailure(#[from] InternalError),
   #[error("JSON deserialisation error: {0}")]
   BadJSON(serde_json::Error),
-  #[error("referenced piece is gone (maybe race)")]
-  PieceHeld,
-  #[error("improper UI operation")]
-  PieceImmoveable,
-  #[error("improper UI operation")]
-  BadOperation,
-  #[error("overlapping occultation")]
-  OverlappingOccultation,
-  #[error("piece is occulting, or occulted")]
-  Occultation,
-  #[error("UI operation not valid in the curret piece state")]
-  BadPieceStateForOperation,
+  #[error("Malformed command - loose not allowed for this op")]
+  BadLoose,
 }
 
 #[derive(Error,Debug)]
@@ -132,41 +120,42 @@ impl From<InternalError> for SpecError {
 
 #[derive(Error,Debug)]
 pub enum ApiPieceOpError {
-  ReportViaResponse(#[from] OnlineError),
-  ReportViaUpdate(#[from] PieceOpError),
+  Fatal(#[from] Fatal),
+  Inapplicable(#[from] Inapplicable),
 
   /// This error is always generated in the context of a piece
   /// operation by a particular client.  It corresponds roughly to a
   /// PieceUpdateFromOp for other clients of (Unpredicable,
   /// PieceUpdateOp::Modify, ..).
   /// For this client it is that but also an error report.
-  PartiallyProcessed(PieceOpError, Vec<LogEntry>),
+  PartiallyProcessed(Inapplicable, Vec<LogEntry>),
 }
 display_as_debug!(ApiPieceOpError);
 
 impl From<PlayerNotFound> for ApiPieceOpError {
   fn from(x: PlayerNotFound) -> ApiPieceOpError {
-    ApiPieceOpError::ReportViaResponse(x.into())
+    ApiPieceOpError::Fatal(x.into())
   }
 }
 impl From<InternalError> for ApiPieceOpError {
   fn from(x: InternalError) -> ApiPieceOpError {
-    ApiPieceOpError::ReportViaResponse(x.into())
+    ApiPieceOpError::Fatal(x.into())
   }
 }
 
-#[derive(Error,Debug,Serialize,Clone)]
-pub enum ErrorSignaledViaUpdate<POEPU: Debug> {
+#[derive(Debug,Serialize,Clone)]
+pub enum ErrorSignaledViaUpdate<POEPU: Debug, EM: Debug> {
   InternalError,
   PlayerRemoved, // appears only in streams for applicable player
   TokenRevoked, // appears only in streams for applicable player
   PieceOpError {
-    error: PieceOpError,
+    error: Inapplicable,
+    error_msg: EM,
     partially: PieceOpErrorPartiallyProcessed,
     state: POEPU,
   },
 }
-display_as_debug!{ErrorSignaledViaUpdate<T>, <T:Debug>}
+//display_as_debug!{ErrorSignaledViaUpdate<T>, <T:Debug>}
 
 #[derive(Error,Debug,Serialize,Copy,Clone,Eq,PartialEq)]
 pub enum PieceOpErrorPartiallyProcessed {
@@ -176,20 +165,26 @@ pub enum PieceOpErrorPartiallyProcessed {
 display_as_debug!{PieceOpErrorPartiallyProcessed}
 
 #[derive(Error,Debug,Serialize,Copy,Clone)]
-pub enum PieceOpError {
-  Conflict,
-  PosOffTable,
-  PieceGone,
-  Occultation,
-  PieceUnrotateable,
-  OcculterAlreadyRotated,
-  OrganisedPlacementOverfull,
+pub enum Inapplicable {
+  #[error("simultaneous update")]           Conflict,
+  #[error("position off table")]            PosOffTable,
+  #[error("piece gone")]                    PieceGone,
+  #[error("piece held by another player")]  PieceHeld,
+  #[error("piece not held by you")]         PieceNotHeld,
+  #[error("prevented by occultation")]      Occultation,
+  #[error("piece may not be rotated")]      PieceUnrotateable,
+  #[error("piece may not be moved")]        PieceImmoveable,
+  #[error("occulter already rotated")]      OcculterAlreadyRotated,
+  #[error("overfull, cannot organise")]     OrganisedPlacementOverfull,
+  #[error("overlapping occultation(s)")]    OverlappingOccultation,
+  #[error("UI operation not recognised")]   BadUiOperation,
+  #[error("UI operation not valid in the curret piece state")]
+                                            BadPieceStateForOperation,
 }
-display_as_debug!{PieceOpError}
 
 pub type StartupError = anyhow::Error;
 
-pub use OnlineError::{NoClient,NoPlayer};
+pub use Fatal::{NoClient,NoPlayer};
 
 pub enum AggregatedIE {
   Ok,
@@ -269,8 +264,8 @@ impl<T> IdForById for T where T: AccessId {
 }
 
 impl IdForById for PieceId {
-  type Error = POE;
-  const ERROR: POE = POE::PieceGone;
+  type Error = Ia;
+  const ERROR: Ia = Ia::PieceGone;
 }
 
 #[macro_export]
