@@ -598,9 +598,9 @@ impl From<&LinksTable> for Html {
 // ---------- Player and token functionality ----------
 
 impl<Id> InstanceAccessDetails<Id>
-  where Id: AccessId, OE: From<Id::Error>
+  where Id: AccessId, Fatal: From<Id::Error>
 {
-  #[throws(OE)]
+  #[throws(Fatal)]
   pub fn from_token(token: &RawTokenVal) -> InstanceAccessDetails<Id> {
     let g = Id::global_tokens(PRIVATE_Y).read();
     let i = g.get(token).ok_or(Id::ERROR)?;
@@ -660,7 +660,7 @@ impl<'ig> InstanceGuard<'ig> {
 
   pub fn remove_clients(&mut self,
                         players: &HashSet<PlayerId>,
-                        signal: ErrorSignaledViaUpdate<PUE_P>) {
+                        signal: ErrorSignaledViaUpdate<PUE_P, String>) {
     let mut clients_to_remove = HashSet::new();
     self.clients.retain(|k,v| {
       let remove = players.contains(&v.player);
@@ -1296,7 +1296,7 @@ pub fn load_games(accounts: &mut AccountsGuard,
 pub type TokenTable<Id> = HashMap<RawToken, InstanceAccessDetails<Id>>;
 
 pub trait AccessId: Copy + Clone + 'static {
-  type Error: Into<OnlineError>;
+  type Error: Into<Fatal>;
   const ERROR: Self::Error;
   fn global_tokens(_:PrivateCaller) -> &'static RwLock<TokenTable<Self>>;
   fn tokens_registry(ig: &mut Instance, _:PrivateCaller)
@@ -1321,8 +1321,8 @@ impl AccessId for PlayerId {
   }
 }
 impl AccessId for ClientId {
-  type Error = OnlineError;
-  const ERROR: OnlineError = NoClient;
+  type Error = Fatal;
+  const ERROR: Fatal = NoClient;
   fn global_tokens(_: PrivateCaller) -> &'static RwLock<TokenTable<Self>> {
     &GLOBAL.clients
   }
@@ -1350,7 +1350,7 @@ pub fn lookup_token<Id:AccessId>(s: &RawTokenVal)
     .ok_or(Id::ERROR)
 }
 
-#[throws(OE)]
+#[throws(Fatal)]
 pub fn record_token<Id:AccessId> (
   ig: &mut InstanceGuard,
   iad: InstanceAccessDetails<Id>
@@ -1411,19 +1411,21 @@ impl GPieces {
   pub fn as_mut(&mut self, _: ModifyingPieces) -> &mut ActualGPieces {
     &mut self.0
   }
+  #[cfg(test)]
+  pub fn as_mut_t(&mut self) -> &mut ActualGPieces { &mut self.0 }
 }
 
 impl ById for GPieces {
   type Id = PieceId;
   type Entry = GPiece;
-  type Error = PieceOpError;
-  #[throws(POE)]
+  type Error = Inapplicable;
+  #[throws(Ia)]
   fn byid(&self, piece: PieceId) -> &GPiece {
-    self.get(piece).ok_or(POE::PieceGone)?
+    self.get(piece).ok_or(Ia::PieceGone)?
   }
-  #[throws(POE)]
+  #[throws(Ia)]
   fn byid_mut(&mut self, piece: PieceId) -> &mut GPiece {
-    self.get_mut(piece).ok_or(POE::PieceGone)?
+    self.get_mut(piece).ok_or(Ia::PieceGone)?
   }
 }
 
