@@ -428,8 +428,8 @@ impl GPiece {
   }
 
   pub fn occulter_check_unrotated(&self, _:ShowUnocculted)
-      -> Result<OcculterRotationChecked, PieceOpError> {
-    if self.angle.is_rotated() { Err(POE::OcculterAlreadyRotated) }
+      -> Result<OcculterRotationChecked, Inapplicable> {
+    if self.angle.is_rotated() { Err(Ia::OcculterAlreadyRotated) }
     else { Ok(OcculterRotationChecked(())) }
   }
 
@@ -455,9 +455,9 @@ impl GPiece {
     self.occult.active.is_some()
   }
 
-  #[throws(OE)]
+  #[throws(Ia)]
   pub fn forbid_involved_in_occultation(&self) {
-    if self.involved_in_occultation() { throw!(OE::Occultation) }
+    if self.involved_in_occultation() { throw!(Ia::Occultation) }
   }
 }
 
@@ -881,7 +881,7 @@ impl OccultationViewDef for OwnerOccultationView {
   } }
 }
 
-#[throws(OnlineError)]
+#[throws(APOE)]
 pub fn create_occultation(
   gen: &mut UniqueGenGen,
   max_z: &mut ZLevel,
@@ -895,7 +895,13 @@ pub fn create_occultation(
   region: Region,
   occulter: PieceId,
   views: OccultationViews,
+  // Caller must promise not to fail if we succeed, so that this
+  // update actually happens!
+  _puos_will_return: &PUOs_Simple_Modify,
 ) -> Vec<(PieceId, PieceUpdateOps)> {
+  // We mustn't actually store this in gpieces until we commit.
+  let ogpc_z_new = piece_make_heavy(gpieces, occulter)?;
+
   {
     let ogpc = gpieces.get(occulter).ok_or_else(
       ||internal_logic_error("create occultation with non-piece"))?;
@@ -917,7 +923,7 @@ pub fn create_occultation(
       // the displ_z, but in case it isn't, we must look at both.
       let max_z = &mut max_z.z;
       (||{
-        max_z.update_max(&ogpc.zlevel.z.clone_mut().increment()?);
+        max_z.update_max(&ogpc_z_new.clone_mut().increment()?);
         max_z.update_max(&displ_z.plus_offset(! 0)?);
         Ok::<_,IE>(())
       })()?;
@@ -925,7 +931,7 @@ pub fn create_occultation(
   }
 
   for occ in goccults.occults.values() {
-    if occ.region.overlaps(&region) { throw!(OE::OverlappingOccultation) }
+    if occ.region.overlaps(&region) { throw!(Ia::OverlappingOccultation) }
   }
 
   let mut recalc = vec![];
@@ -958,6 +964,7 @@ pub fn create_occultation(
       let ogpc = gpieces.get_mut(occulter).ok_or_else(
         ||internal_logic_error("occulter vanished"))?;
       ogpc.occult.active = Some(occid);
+      ogpc.zlevel.z = ogpc_z_new;
 
       for &ppiece in &recalc {
         recalculate_occultation_ofmany(gen,
