@@ -202,6 +202,12 @@ impl<R:Read> FrameReader<R> {
     Some(ReadFrame { fr: self })
   }
 
+  #[throws(io::Error)]
+  pub fn into_stream(mut self) -> BufReader<Fuse<R>> {
+    self.finish_reading_frame()?;
+    self.inner
+  }
+
   #[throws(io::Error)]
   fn finish_reading_frame(&mut self) {
     while matches_doesnot!(
@@ -357,6 +363,12 @@ impl<W:Write> FrameWriter<W> {
     FrameWriter { inner: Fuse::new(w), in_frame: None }
   }
 
+  #[throws(io::Error)]
+  pub fn into_stream(mut self) -> Fuse<W> {
+    self.tidy()?;
+    self.inner
+  }
+
   #[throws(io::Error)]
   pub fn new_frame<'w>(&'w mut self) -> WriteFrame<'w,W> {
     self.tidy()?;
