@@ -68,7 +68,7 @@ pub use flexi_logger::LogSpecification;
 pub use fs2::FileExt;
 pub use if_chain::if_chain;
 pub use index_vec::{define_index_type, index_vec, IndexSlice, IndexVec};
-pub use itertools::{izip, zip_eq, EitherOrBoth, Itertools};
+pub use itertools::{iproduct, izip, zip_eq, EitherOrBoth, Itertools};
 pub use lazy_static::lazy_static;
 pub use log::{debug, error, info, trace, warn};
 pub use log::{log, log_enabled};
@@ -95,7 +95,7 @@ pub use serde_with::DeserializeFromStr;
 pub use serde_with::SerializeDisplay;
 pub use sha2::{Sha512, Sha512Trunc256};
 pub use slotmap::{dense::DenseSlotMap, SparseSecondaryMap, Key as _};
-pub use strum::{EnumCount};
+pub use strum::{EnumCount, EnumDiscriminants};
 pub use strum::{EnumString, EnumIter, EnumMessage, EnumProperty};
 pub use strum::{IntoEnumIterator, IntoStaticStr};
 pub use subtle::ConstantTimeEq;
@@ -115,9 +115,9 @@ pub use otter_base::geometry::{self,Coord,Pos,PosC,Rect,RectC};
 pub use otter_base::geometry::{CoordinateOverflow,Region};
 pub use otter_base::zcoord::{self, ZCoord};
 pub use otter_base::misc as base_misc;
+pub use otter_base::dbgc;
 pub use base_misc::*;
 
-pub use crate::dbgc;
 pub use crate::{deref_to_field, deref_to_field_mut};
 pub use crate::ensure_eq;
 pub use crate::format_by_fmt_hex;
@@ -194,10 +194,9 @@ pub type PUM = ProgressUpdateMode;
 
 // error.rs
 pub type APOE = ApiPieceOpError;
-pub type ESVU<POEPU> = ErrorSignaledViaUpdate<POEPU>;
+pub type ESVU<POEPU,EM> = ErrorSignaledViaUpdate<POEPU,EM>;
 pub type IE = InternalError;
-pub type OE = OnlineError;
-pub type POE = PieceOpError;
+pub type Ia = Inapplicable;
 pub type POEPP = PieceOpErrorPartiallyProcessed;
 pub type SvgE = SVGProcessingError;
 pub type SpE = SpecError;
