@@ -74,7 +74,7 @@ pub enum PreparedUpdateEntry {
     new_info_pane: Arc<Html>,
   },
   Log(Arc<CommittedLogEntry>),
-  Error(ErrorSignaledViaUpdate<PUE_P>),
+  Error(ErrorSignaledViaUpdate<PUE_P, String>),
 }
 
 #[allow(non_camel_case_types)]
@@ -213,6 +213,14 @@ impl From<PieceUpdateFromOpSimple> for PieceUpdate {
   }
 }
 
+
+#[allow(non_camel_case_types)]
+#[derive(Copy,Clone,Debug)]
+pub struct PUOs_Simple_Modify;
+impl From<PUOs_Simple_Modify> for PieceUpdateOps {
+  fn from(_: PUOs_Simple_Modify) -> Self { PUOs::Simple(PUO::Modify(())) }
+}
+
 // ---------- for traansmission ----------
 
 #[derive(Debug,Serialize)]
@@ -255,11 +263,21 @@ enum TransmitUpdateEntry<'u> {
   SetLinks(Html),
   #[serde(serialize_with="serialize_logentry")]
   Log(TransmitUpdateLogEntry<'u>),
-  Error(ErrorSignaledViaUpdate<TUE_P<'u>>),
+  Error(ErrorSignaledViaUpdate<ETUE_P<'u>, &'u str>),
 }
 
 type TransmitUpdateLogEntry<'u> = (&'u Timezone, &'u CommittedLogEntry);
 
+#[allow(non_camel_case_types)]
+#[derive(Debug,Serialize)]
+struct ErrorTransmitUpdateEntry_Piece<'u> {
+  cseq: Option<ClientSequence>,
+  #[serde(flatten)]
+  tue: TransmitUpdateEntry_Piece<'u>,
+}
+#[allow(non_camel_case_types)]
+type ETUE_P<'u> = ErrorTransmitUpdateEntry_Piece<'u>;
+
 #[allow(non_camel_case_types)]
 #[derive(Debug,Serialize)]
 struct TransmitUpdateEntry_Piece<'u> {
@@ -284,7 +302,7 @@ struct FormattedLogEntry<'u> {
 
 // ---------- helpful utilities ----------
 
-#[throws(OE)]
+#[throws(Fatal)]
 pub fn log_did_to_piece_whoby(ioccults: &IOccults, goccults: &GameOccults,
                               by_gpl: &GPlayer,
                               gpc: &GPiece, ipc: &IPiece, did: &str)
@@ -311,7 +329,7 @@ pub fn log_did_to_piece_whoby(ioccults: &IOccults, goccults: &GameOccults,
   (log, Some(who_by))
 }
 
-#[throws(OE)]
+#[throws(Fatal)]
 pub fn log_did_to_piece(ioccults: &IOccults, goccults: &GameOccults,
                         by_gpl: &GPlayer,
                         gpc: &GPiece, ipc: &IPiece, did: &str)
@@ -611,11 +629,11 @@ impl<'r> PrepareUpdatesBuffer<'r> {
   }
 
   pub fn piece_report_error(ig: &mut Instance,
-                            error: PieceOpError, piece: PieceId,
+                            error: Inapplicable, piece: PieceId,
                             logents: Vec<LogEntry>,
                             partially: PieceOpErrorPartiallyProcessed,
                             client: ClientId, cseq: ClientSequence)
-                            -> Result<(),OE> {
+                            -> Result<(),Fatal> {
     let by_client = (WRC::Unpredictable, client, cseq);
     let mut buf = PrepareUpdatesBuffer::new(ig, None);
     let ops = PUOs::Simple(PieceUpdateOp::Modify(()));
@@ -628,7 +646,8 @@ impl<'r> PrepareUpdatesBuffer<'r> {
       }
     )?;
     let update = PUE::Error(ESVU::PieceOpError {
-      error, state, partially
+      error, state, partially,
+      error_msg: error.to_string(),
     });
     buf.us.push(update);
     buf.log_updates(logents);
@@ -870,7 +889,7 @@ impl PreparedUpdate {
   pub fn for_transmit<'u>(&'u self, tz: &'u Timezone,
                           player: PlayerId, dest: ClientId)
                           -> TransmitUpdate<'u> {
-    type ESVU<T> = ErrorSignaledViaUpdate<T>;
+    type ESVU<T,EM> = ErrorSignaledViaUpdate<T,EM>;
     type PUE = PreparedUpdateEntry;
     type TUE<'u> = TransmitUpdateEntry<'u>;
     let mut ents = vec![];
@@ -970,15 +989,23 @@ impl PreparedUpdate {
             ESVU::InternalError => TUE::Error(ESVU::InternalError),
             ESVU::PlayerRemoved => TUE::Error(ESVU::PlayerRemoved),
             ESVU::TokenRevoked  => TUE::Error(ESVU::TokenRevoked),
-            ESVU::PieceOpError { error, partially, ref state } => {
-              let c = state.by_client.as_ref().map(|(_,c,_)| *c);
+            ESVU::PieceOpError { error, partially,
+                                 ref error_msg, ref state } => {
+              let c    = state.by_client.as_ref().map(|(_,c,_   )| *c);
+              let cseq = state.by_client.as_ref().map(|(_,_,cseq)| *cseq);
               if c == None || c == Some(dest) {
-                let state = match pue_piece_to_tue_p(state, player) {
+                let tue = match pue_piece_to_tue_p(state, player) {
                   Some(tue) => tue,
                   None => continue,
                 };
                 TUE::Error(
-                  ESVU::PieceOpError { error, partially, state }
+                  ESVU::PieceOpError {
+                    error, error_msg, partially,
+                    state: { ETUE_P {
+                      tue,
+                      cseq,
+                    } },
+                  }
                 )
               } else {
                 match partially {
